package com.loafle.overflow.central.module.sensor.service;

import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.zip.GZIPOutputStream;

import javax.crypto.Cipher;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.transaction.Transactional;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.loafle.overflow.central.commons.service.MessagePublisher;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraHost;
import com.loafle.overflow.model.infra.InfraHostPort;
import com.loafle.overflow.model.infra.InfraService;
import com.loafle.overflow.model.infra.InfraZone;
import com.loafle.overflow.model.meta.MetaCollectionItem;
import com.loafle.overflow.model.meta.MetaCollectionItemMapping;
import com.loafle.overflow.model.meta.MetaCryptoType;
import com.loafle.overflow.model.meta.MetaInfraType;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;
import com.loafle.overflow.model.sensorconfig.SensorConfig;
import com.loafle.overflow.model.sensorconfig.SensorConfigConnection;
import com.loafle.overflow.model.sensorconfig.SensorConfigCrawler;
import com.loafle.overflow.model.sensorconfig.SensorConfigSchedule;
import com.loafle.overflow.service.central.meta.MetaCollectionItemMappingService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("SensorConfigService")
public class CentralSensorConfigService {

  @Autowired
  MetaCollectionItemMappingService metaCollectionItemMappingService;
  @Autowired
  CentralSensorItemService sensorItemService;
  @Autowired
  ObjectMapper objectMapper;
  @Autowired
  private MessagePublisher messagePublisher;

  @Transactional
  public SensorConfig regist(Sensor sensor, boolean update) throws OverflowException {
    if (null == sensor.getId()) {
      throw new OverflowException(String.format("ID[%s] of Sensor is not valid", sensor.getId()));
    }
    SensorConfig sensorConfig = new SensorConfig();
    sensorConfig.setSensorID(sensor.getId());
    sensorConfig.setCrawler(this.configCrawler(sensor));
    sensorConfig.setConnection(this.configConnection(sensor));
    sensorConfig.setSchedule(this.configSchedule(sensor));
    sensorConfig.setMetaCollectionItems(this.configItems(sensor));

    Probe probe = sensor.getTarget().getInfra().getProbe();
    String encryptedSensorConfigBase64 = this.toEncryptString(sensorConfig, probe.getEncryptionKey());
    String method = update ? "SensorService.UpdateSensor" : "SensorService.AddSensor";
    this.messagePublisher.publishToProbe(probe.getProbeKey(), method, encryptedSensorConfigBase64);

    return sensorConfig;
  }

  private String toEncryptString(SensorConfig sensorConfig, String encryptKey) throws OverflowException {

    try {
      String sensorConfigJsonStr = this.objectMapper.writeValueAsString(sensorConfig);
      // gzip
      ByteArrayOutputStream bos = new ByteArrayOutputStream(sensorConfigJsonStr.length());
      GZIPOutputStream gzip = new GZIPOutputStream(bos);
      gzip.write(sensorConfigJsonStr.getBytes("UTF-8"));
      gzip.flush();
      gzip.close();
      byte[] compressedByte = bos.toByteArray();
      bos.close();

      // DES encryption
      Cipher cipher = Cipher.getInstance("DES/ECB/PKCS5Padding");
      SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
      DESKeySpec desKeySpec = new DESKeySpec(encryptKey.getBytes());
      cipher.init(Cipher.ENCRYPT_MODE, keyFactory.generateSecret(desKeySpec));
      byte[] outputBytes = cipher.doFinal(compressedByte);

      // Base64
      return Base64.getEncoder().encodeToString(outputBytes);
    } catch (Exception e) {
      throw new OverflowException("", e);
    }

  }

  /*
   * private String testDecrypt(String encryptKey, String encrypted) throws
   * OverflowException { try { byte[] inputBytes =
   * Base64.getDecoder().decode(encrypted);
   *
   * Cipher cipher = Cipher.getInstance("DES"); SecretKeyFactory keyFactory =
   * SecretKeyFactory.getInstance("DES"); DESKeySpec desKeySpec = new
   * DESKeySpec(encryptKey.getBytes()); cipher.init(Cipher.DECRYPT_MODE,
   * keyFactory.generateSecret(desKeySpec)); byte[] outputBytes =
   * cipher.doFinal(inputBytes);
   *
   * GZIPInputStream gis = new GZIPInputStream(new
   * ByteArrayInputStream(outputBytes)); BufferedReader bf = new
   * BufferedReader(new InputStreamReader(gis, "UTF-8")); String result = "";
   * String line; while ((line = bf.readLine()) != null) { result += line; }
   *
   * return result;
   *
   * } catch (Exception e) { throw new OverflowException("", e); } }
   */

  private SensorConfigCrawler configCrawler(Sensor sensor) throws OverflowException {
    if (null == sensor.getMetaCrawlerMapping() || null == sensor.getMetaCrawlerMapping().getMetaCrawler()
        || null == sensor.getMetaCrawlerMapping().getMetaCrawlerContainer()) {
      throw new OverflowException(String.format("ID[%s] of Sensor is not valid", sensor.getId()));
    }
    SensorConfigCrawler sensorConfigCrawler = new SensorConfigCrawler();
    sensorConfigCrawler.setMetaCrawlerContainerKey(sensor.getMetaCrawlerMapping().getMetaCrawlerContainer().getKey());
    sensorConfigCrawler.setMetaCrawlerKey(sensor.getMetaCrawlerMapping().getMetaCrawler().getKey());

    return sensorConfigCrawler;
  }

  private SensorConfigConnection configConnection(Sensor sensor) throws OverflowException {
    if (null == sensor.getTarget() || null == sensor.getTarget().getInfra()
        || null == sensor.getTarget().getInfra().getMetaInfraType()) {
      throw new OverflowException(String.format("ID[%s] of Sensor is not valid", sensor.getId()));
    }
    Infra infra = sensor.getTarget().getInfra();
    MetaInfraType metaInfraType = infra.getMetaInfraType();
    String InfraHostIPAddress = null;
    String metaIPTypeKey = null;
    InfraHostPort infraHostPort = null;
    MetaCryptoType metaCryptoType = null;

    if (metaInfraType.getKey().equals("HOST")) {
      InfraHost infraHost = (InfraHost) infra;
      metaIPTypeKey = infraHost.getInfraHostIPs().get(0).getMetaIPType().getKey();
      InfraHostIPAddress = infraHost.getInfraHostIPs().get(0).getAddress();

    } else if (metaInfraType.getKey().equals("SERVICE")) {
      InfraService infraService = (InfraService) infra;
      metaIPTypeKey = infraService.getInfraHostPort().getInfraHostIP().getMetaIPType().getKey();
      InfraHostIPAddress = infraService.getInfraHostPort().getInfraHostIP().getAddress();
      infraHostPort = infraService.getInfraHostPort();
      metaCryptoType = infraService.getMetaCryptoType();

    } else if (metaInfraType.getKey().equals("ZONE")) {
      InfraZone infraZone = (InfraZone) infra;
      InfraHostIPAddress = infraZone.getAddress();
      metaIPTypeKey = infraZone.getMetaIPType().getKey();
    } else {
      throw new OverflowException(String.format("ID[%s] of Sensor is not valid", sensor.getId()));
    }

    SensorConfigConnection sensorConfigConnection = new SensorConfigConnection();
    sensorConfigConnection.setMetaIPTypeKey(metaIPTypeKey);
    sensorConfigConnection.setIp(InfraHostIPAddress);
    if (null != infraHostPort) {
      sensorConfigConnection.setMetaPortTypeKey(infraHostPort.getMetaPortType().getKey());
      sensorConfigConnection.setPort(infraHostPort.getPort());
    }
    if (null != metaCryptoType) {
      sensorConfigConnection.setMetaCryptoTypeKey(metaCryptoType.getKey());
    }

    return sensorConfigConnection;
  }

  private SensorConfigSchedule configSchedule(Sensor sensor) throws OverflowException {
    Integer interval = null != sensor.getInterval() ? sensor.getInterval()
        : sensor.getMetaCrawlerMapping().getDefaultInterval();
    SensorConfigSchedule sensorConfigSchedule = new SensorConfigSchedule();
    sensorConfigSchedule.setInterval(interval);
    return sensorConfigSchedule;
  }

  private List<MetaCollectionItem> configItems(Sensor sensor) throws OverflowException {
    List<SensorItem> sensorItems = this.sensorItemService.readAllBySensorID(sensor.getId());
    if (null == sensorItems || sensorItems.size() == 0) {
      throw new OverflowException(String.format("ID[%s] of Sensor has no SensorItem", sensor.getId()));
    }
    List<MetaCollectionItem> metaCollectionItems = new ArrayList<>();
    for (SensorItem sensorItem : sensorItems) {
      List<MetaCollectionItemMapping> metaCollectionItemMappings = this.metaCollectionItemMappingService
          .readAllByMetaDisplayItemMappingID(sensorItem.getMetaDisplayItemMapping().getId());
      for (MetaCollectionItemMapping metaCollectionItemMapping : metaCollectionItemMappings) {
        metaCollectionItems.add(metaCollectionItemMapping.getMetaCollectionItem());
      }
    }

    return metaCollectionItems;
  }
}
